@@ -1,6 +1,3 @@
-# File modified under the Apache Licence 2.0 spec. Modified by Aero Technologies.
-# Updated how CLI is built
-
 from __future__ import print_function
 
 import time
@@ -14,7 +11,6 @@ from tempfile import NamedTemporaryFile
 from multiprocessing import Process, Queue
 from collections import namedtuple
 from itertools import starmap, chain, islice
-from botocore.exceptions import ClientError
 
 try:
     # python2
@@ -313,226 +309,227 @@ def parallel_op(op, lst, num_workers):
         for x in chain.from_iterable(it):
             yield x
 
+# CLI
 
-def create_s3_cli(cli):
+@click.group()
+def cli():
+    pass
 
+@cli.command('list', help='List S3 objects')
+@click.option('--inputs',
+              type=click.Path(exists=True),
+              help='Read input prefixes from the given file.')
+@click.option('--num-workers',
+              default=NUM_WORKERS_DEFAULT,
+              show_default=True,
+              help='Number of concurrent connections.')
+@click.option('--recursive/--no-recursive',
+              default=False,
+              show_default=True,
+              help='Download prefixes recursively.')
+@click.argument('prefixes', nargs=-1)
+def lst(prefixes,
+        inputs=None,
+        num_workers=None,
+        recursive=None):
 
-    @cli.group(help="S3 Interface")
-    def s3():
-        pass
+    urllist = []
+    for prefix in _populate_prefixes(prefixes, inputs):
+        src = urlparse(prefix)
+        url = S3Url(url=prefix,
+                    bucket=src.netloc,
+                    path=src.path.lstrip('/'),
+                    local=None,
+                    prefix=prefix)
+        if src.scheme != 's3':
+            exit(ERROR_INVALID_URL, url)
+        urllist.append(url)
 
-    @s3.command('list', help='List S3 objects')
-    @click.option('--inputs',
-                type=click.Path(exists=True),
-                help='Read input prefixes from the given file.')
-    @click.option('--num-workers',
-                default=NUM_WORKERS_DEFAULT,
-                show_default=True,
-                help='Number of concurrent connections.')
-    @click.option('--recursive/--no-recursive',
-                default=False,
-                show_default=True,
-                help='Download prefixes recursively.')
-    @click.argument('prefixes', nargs=-1)
-    def lst(prefixes,
-            inputs=None,
-            num_workers=None,
-            recursive=None):
+    op = op_list_prefix if recursive else op_list_prefix_nonrecursive
+    urls = []
+    for success, prefix_url, ret in parallel_op(op, urllist, num_workers):
+        if success:
+            urls.extend(ret)
+        else:
+            exit(ret, prefix_url)
 
-        urllist = []
-        for prefix in _populate_prefixes(prefixes, inputs):
-            src = urlparse(prefix)
-            url = S3Url(url=prefix,
-                        bucket=src.netloc,
-                        path=src.path.lstrip('/'),
-                        local=None,
-                        prefix=prefix)
-            if src.scheme != 's3':
-                exit(ERROR_INVALID_URL, url)
-            urllist.append(url)
+    for url, size in urls:
+        if size is None:
+            print(format_triplet(url.prefix, url.url))
+        else:
+            print(format_triplet(url.prefix, url.url, str(size)))
 
-        op = op_list_prefix if recursive else op_list_prefix_nonrecursive
+@cli.command(help='Upload files to S3')
+@click.option('--file',
+              'files',
+              type=(click.Path(exists=True), str),
+              multiple=True,
+              help='Local file->S3Url pair to upload. '
+                   'Can be specified multiple times.')
+@click.option('--filelist',
+              type=click.Path(exists=True),
+              help='Read local file -> S3 URL mappings from the given file.')
+@click.option('--num-workers',
+              default=NUM_WORKERS_DEFAULT,
+              show_default=True,
+              help='Number of concurrent connections.')
+@click.option('--verbose/--no-verbose',
+              default=True,
+              show_default=True,
+              help='Print status information on stderr.')
+@click.option('--overwrite/--no-overwrite',
+              default=True,
+              show_default=True,
+              help='Overwrite key if it already exists in S3.')
+@click.option('--listing/--no-listing',
+              default=False,
+              show_default=True,
+              help='Print S3 URLs upload to on stdout.')
+def put(files=None,
+        filelist=None,
+        num_workers=None,
+        verbose=None,
+        overwrite=True,
+        listing=None):
+
+    def _files():
+        for local, url in files:
+            yield url_unquote(local), url_unquote(url)
+        if filelist:
+            for line in open(filelist, mode='rb'):
+                local, url = map(url_unquote, line.split())
+                if not os.path.exists(local):
+                    exit(ERROR_LOCAL_FILE_NOT_FOUND, local)
+                yield local, url
+
+    def _make_url(local, user_url):
+        src = urlparse(user_url)
+        url = S3Url(url=user_url,
+                    bucket=src.netloc,
+                    path=src.path.lstrip('/'),
+                    local=local,
+                    prefix=None)
+        if src.scheme != 's3':
+            exit(ERROR_INVALID_URL, url)
+        if not src.path:
+            exit(ERROR_NOT_FULL_PATH, url)
+        return url, os.stat(local).st_size
+
+    urls = list(starmap(_make_url, _files()))
+    if not overwrite:
+        new_urls = set()
+        for success, prefix_url, ret in parallel_op(op_get_size, list(list(zip(*urls))[0]), num_workers):
+            if ret == ERROR_URL_NOT_FOUND:
+                new_urls.add(prefix_url)
+        urls = [(url, size) for url, size in urls if url in new_urls]
+    process_urls('upload', urls, verbose, num_workers)
+    if listing:
+        for url, _ in urls:
+            print(format_triplet(url.url))
+
+def _populate_prefixes(prefixes, inputs):
+    if not prefixes:
+        prefixes = []
+    if inputs:
+        with open(inputs, mode='rb') as f:
+            prefixes.extend(l.strip() for l in f)
+    return list(map(url_unquote, prefixes))
+
+@cli.command(help='Download files from S3')
+@click.option('--recursive/--no-recursive',
+              default=False,
+              show_default=True,
+              help='Download prefixes recursively.')
+@click.option('--num-workers',
+              default=NUM_WORKERS_DEFAULT,
+              show_default=True,
+              help='Number of concurrent connections.')
+@click.option('--inputs',
+              type=click.Path(exists=True),
+              help='Read input prefixes from the given file.')
+@click.option('--verify/--no-verify',
+              default=True,
+              show_default=True,
+              help='Verify that files were loaded correctly.')
+@click.option('--allow-missing/--no-allow-missing',
+              default=False,
+              show_default=True,
+              help='Do not exit if missing files are detected. '\
+                   'Implies --verify.')
+@click.option('--verbose/--no-verbose',
+              default=True,
+              show_default=True,
+              help='Print status information on stderr.')
+@click.option('--listing/--no-listing',
+              default=False,
+              show_default=True,
+              help='Print S3 URL -> local file mapping on stdout.')
+@click.argument('prefixes', nargs=-1)
+def get(prefixes,
+        recursive=None,
+        num_workers=None,
+        inputs=None,
+        verify=None,
+        allow_missing=None,
+        verbose=None,
+        listing=None):
+
+    if allow_missing:
+        verify = True
+
+    # Construct a list of URL (prefix) objects
+    urllist = []
+    for prefix in _populate_prefixes(prefixes, inputs):
+        src = urlparse(prefix)
+        url = S3Url(url=prefix,
+                    bucket=src.netloc,
+                    path=src.path.lstrip('/'),
+                    local=generate_local_path(prefix),
+                    prefix=prefix)
+        if src.scheme != 's3':
+            exit(ERROR_INVALID_URL, url)
+        if not recursive and not src.path:
+            exit(ERROR_NOT_FULL_PATH, url)
+        urllist.append(url)
+
+    # Construct a url->size mapping
+    op = None
+    if recursive:
+        op = op_list_prefix
+    elif verify or verbose:
+        op = op_get_size
+    if op:
         urls = []
+
+        # NOTE - we must retain the order of prefixes requested
+        # and the listing order returned by S3
         for success, prefix_url, ret in parallel_op(op, urllist, num_workers):
             if success:
                 urls.extend(ret)
+            elif ret == ERROR_URL_NOT_FOUND and allow_missing:
+                urls.append((prefix_url, None))
             else:
                 exit(ret, prefix_url)
+    else:
+        # pretend zero size since we don't need it for anything.
+        # it can't be None though, to make sure the listing below
+        # works correctly (None denotes a missing file)
+        urls = [(prefix_url, 0) for prefix_url in urllist]
 
+    # exclude the non-existent files from loading
+    to_load = [(url, size) for url, size in urls if size is not None]
+    process_urls('download', to_load, verbose, num_workers)
+    # Postprocess
+    if verify:
+        verify_results(to_load, verbose=verbose)
+
+    if listing:
         for url, size in urls:
             if size is None:
-                print(format_triplet(url.prefix, url.url))
-            else:
-                print(format_triplet(url.prefix, url.url, str(size)))
-
-    @s3.command(help='Upload files to S3')
-    @click.option('--file',
-                'files',
-                type=(click.Path(exists=True), str),
-                multiple=True,
-                help='Local file->S3Url pair to upload. '
-                    'Can be specified multiple times.')
-    @click.option('--filelist',
-                type=click.Path(exists=True),
-                help='Read local file -> S3 URL mappings from the given file.')
-    @click.option('--num-workers',
-                default=NUM_WORKERS_DEFAULT,
-                show_default=True,
-                help='Number of concurrent connections.')
-    @click.option('--verbose/--no-verbose',
-                default=True,
-                show_default=True,
-                help='Print status information on stderr.')
-    @click.option('--overwrite/--no-overwrite',
-                default=True,
-                show_default=True,
-                help='Overwrite key if it already exists in S3.')
-    @click.option('--listing/--no-listing',
-                default=False,
-                show_default=True,
-                help='Print S3 URLs upload to on stdout.')
-    def put(files=None,
-            filelist=None,
-            num_workers=None,
-            verbose=None,
-            overwrite=True,
-            listing=None):
-
-        def _files():
-            for local, url in files:
-                yield url_unquote(local), url_unquote(url)
-            if filelist:
-                for line in open(filelist, mode='rb'):
-                    local, url = map(url_unquote, line.split())
-                    if not os.path.exists(local):
-                        exit(ERROR_LOCAL_FILE_NOT_FOUND, local)
-                    yield local, url
-
-        def _make_url(local, user_url):
-            src = urlparse(user_url)
-            url = S3Url(url=user_url,
-                        bucket=src.netloc,
-                        path=src.path.lstrip('/'),
-                        local=local,
-                        prefix=None)
-            if src.scheme != 's3':
-                exit(ERROR_INVALID_URL, url)
-            if not src.path:
-                exit(ERROR_NOT_FULL_PATH, url)
-            return url, os.stat(local).st_size
-
-        urls = list(starmap(_make_url, _files()))
-        if not overwrite:
-            new_urls = set()
-            for success, prefix_url, ret in parallel_op(op_get_size, list(list(zip(*urls))[0]), num_workers):
-                if ret == ERROR_URL_NOT_FOUND:
-                    new_urls.add(prefix_url)
-            urls = [(url, size) for url, size in urls if url in new_urls]
-        process_urls('upload', urls, verbose, num_workers)
-        if listing:
-            for url, _ in urls:
                 print(format_triplet(url.url))
+            else:
+                print(format_triplet(url.prefix, url.url, url.local))
 
-    def _populate_prefixes(prefixes, inputs):
-        if not prefixes:
-            prefixes = []
-        if inputs:
-            with open(inputs, mode='rb') as f:
-                prefixes.extend(l.strip() for l in f)
-        return list(map(url_unquote, prefixes))
-
-    @s3.command(help='Download files from S3')
-    @click.option('--recursive/--no-recursive',
-                default=False,
-                show_default=True,
-                help='Download prefixes recursively.')
-    @click.option('--num-workers',
-                default=NUM_WORKERS_DEFAULT,
-                show_default=True,
-                help='Number of concurrent connections.')
-    @click.option('--inputs',
-                type=click.Path(exists=True),
-                help='Read input prefixes from the given file.')
-    @click.option('--verify/--no-verify',
-                default=True,
-                show_default=True,
-                help='Verify that files were loaded correctly.')
-    @click.option('--allow-missing/--no-allow-missing',
-                default=False,
-                show_default=True,
-                help='Do not exit if missing files are detected. '\
-                    'Implies --verify.')
-    @click.option('--verbose/--no-verbose',
-                default=True,
-                show_default=True,
-                help='Print status information on stderr.')
-    @click.option('--listing/--no-listing',
-                default=False,
-                show_default=True,
-                help='Print S3 URL -> local file mapping on stdout.')
-    @click.argument('prefixes', nargs=-1)
-    def get(prefixes,
-            recursive=None,
-            num_workers=None,
-            inputs=None,
-            verify=None,
-            allow_missing=None,
-            verbose=None,
-            listing=None):
-
-        if allow_missing:
-            verify = True
-
-        # Construct a list of URL (prefix) objects
-        urllist = []
-        for prefix in _populate_prefixes(prefixes, inputs):
-            src = urlparse(prefix)
-            url = S3Url(url=prefix,
-                        bucket=src.netloc,
-                        path=src.path.lstrip('/'),
-                        local=generate_local_path(prefix),
-                        prefix=prefix)
-            if src.scheme != 's3':
-                exit(ERROR_INVALID_URL, url)
-            if not recursive and not src.path:
-                exit(ERROR_NOT_FULL_PATH, url)
-            urllist.append(url)
-
-        # Construct a url->size mapping
-        op = None
-        if recursive:
-            op = op_list_prefix
-        elif verify or verbose:
-            op = op_get_size
-        if op:
-            urls = []
-
-            # NOTE - we must retain the order of prefixes requested
-            # and the listing order returned by S3
-            for success, prefix_url, ret in parallel_op(op, urllist, num_workers):
-                if success:
-                    urls.extend(ret)
-                elif ret == ERROR_URL_NOT_FOUND and allow_missing:
-                    urls.append((prefix_url, None))
-                else:
-                    exit(ret, prefix_url)
-        else:
-            # pretend zero size since we don't need it for anything.
-            # it can't be None though, to make sure the listing below
-            # works correctly (None denotes a missing file)
-            urls = [(prefix_url, 0) for prefix_url in urllist]
-
-        # exclude the non-existent files from loading
-        to_load = [(url, size) for url, size in urls if size is not None]
-        process_urls('download', to_load, verbose, num_workers)
-        # Postprocess
-        if verify:
-            verify_results(to_load, verbose=verbose)
-
-        if listing:
-            for url, size in urls:
-                if size is None:
-                    print(format_triplet(url.url))
-                else:
-                    print(format_triplet(url.prefix, url.url, url.local))
-
+if __name__ == '__main__':
+    from botocore.exceptions import ClientError
+    cli(auto_envvar_prefix='S3OP')
